// HudOverlay.h : Declaration of the CHudOverlay

#ifndef __HUDOVERLAY_H_
#define __HUDOVERLAY_H_

#include "resource.h"       // main symbols
#include "IadsVideoOverlay.h"

static HWND gDialogHwnd;
static char gLabelProperty[500];
static bool gUserCanceled = true;

/////////////////////////////////////////////////////////////////////////////
// CHudOverlay
class ATL_NO_VTABLE CHudOverlay : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CHudOverlay, &CLSID_HudOverlay>,
	public IDispatchImpl<IHudOverlay, &IID_IHudOverlay, &LIBID_SAMPLEVIDEOSERVEROVERLAYLib>,
  public IIadsVideoOverlay,
  public IPersistPropertyBag
{
public:
  char mMsg[500];

	CHudOverlay()
	{
    mMsg[0] = '\0';
	}

DECLARE_REGISTRY_RESOURCEID(IDR_HUDOVERLAY)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CHudOverlay)
	COM_INTERFACE_ENTRY(IHudOverlay)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IIadsVideoOverlay)
	COM_INTERFACE_ENTRY(IPersistPropertyBag)
END_COM_MAP()

// IHudOverlay  Run the IadsAudioVideoServer with the following command line to activate the overlay:
//      /Overlay SampleVideoServerOverlay.HudOverlay /server yourCDSserverHostName
// If you need to test in PostTest, you can run your PTDS with the command line option   
//      /portId 58000
// and then run the IadsAVServer with:
//      /Overlay SampleVideoServerOverlay.HudOverlay /server yourPTDSserverHostName
public:

  // IIadsVideoOverlay
  //[id(1), helpstring("Your plugin should return a comma separated list of parameter needed when this function called")] 
  STDMETHOD(GetInputParameterList)( BSTR* commaSepListOfParameters )
  {
    if ( !commaSepListOfParameters ) return E_POINTER;
    *commaSepListOfParameters = ::SysAllocString( L"SineWave1,SineWave2" );
    return *commaSepListOfParameters ? S_OK : E_OUTOFMEMORY;
  }

  // When the Server calls this function, you'll be given the DC to draw into and a 
  // CVT of the input parameter values requested at the time of the video frame
  //[id(2), helpstring("Your plugin should return a comma separated list of parameter needed when this function called")] 
  STDMETHOD(DrawOverlay)( HDC DCToDraw, int dcWidth, int dcHeight, double* cvtArrayOfDoubles, int numCvtValues )
  {
    if ( !cvtArrayOfDoubles ) return E_POINTER;
    if ( numCvtValues < 2 ) return E_UNEXPECTED;

    register double s1 = cvtArrayOfDoubles[0];
    register double s2 = cvtArrayOfDoubles[1];

    RECT r = { 0, 0, dcWidth, dcHeight };
    char msg[1000];
    sprintf( msg, "%s SineWave1=% 0.3f  SineWave2=% 0.3f", mMsg, s1, s2 );
    HPEN pen = ::CreatePen( PS_SOLID, 1, RGB(255,0,0) );
    HPEN op = (HPEN)::SelectObject( DCToDraw, pen );
    ::DrawText( DCToDraw, msg, strlen(msg), &r, DT_BOTTOM );
    ::MoveToEx( DCToDraw, 0, 0, NULL );
    ::LineTo( DCToDraw, dcWidth-1, dcHeight-1 );
    ::MoveToEx( DCToDraw, dcWidth-1, 0, NULL );
    ::LineTo( DCToDraw, 0, dcHeight-1 );
    ::SelectObject( DCToDraw, op );
    ::DeleteObject( pen );
    return S_OK;
  }

  static BOOL CALLBACK DialogCallback( HWND handle, UINT msg, WPARAM wParam, LPARAM lParam )
  {
    switch( msg )
    {
      case WM_INITDIALOG :
      {
        gDialogHwnd = handle;
        ::SetDlgItemText( gDialogHwnd, IDC_EDIT1, gLabelProperty );
        return TRUE;
      }

      case WM_SYSCOMMAND:
      {
        if( wParam != SC_CLOSE ) break;
        ::EndDialog( handle, 0 );
      }

      case WM_COMMAND:
        switch ( LOWORD(wParam) )
        {
          case IDOK:
            // User hit Ok on the dialog. Extract the values from the GUI and set your member variables
            ::GetDlgItemText( gDialogHwnd, IDC_EDIT1, gLabelProperty, 499 );
            gUserCanceled = false;
            ::EndDialog( handle, 0 );
            break;
          case IDCANCEL:
            gUserCanceled = true;
            ::EndDialog( handle, 0 );
            break;
        }
      break;
    }
    return FALSE;
  }

  // The user selected the "show overlay properties" option from the menu. Time to bring up your properties dialog.
  STDMETHOD(ShowPropertiesDialog)( /*[in]*/HWND ParentHwnd )
  {
    gUserCanceled = true;
    strcpy( gLabelProperty, mMsg );

    ::DialogBox( _Module.GetResourceInstance(), MAKEINTRESOURCE(IDD_PROPDLG), ParentHwnd, DialogCallback );
    
    if ( !gUserCanceled )
    {
      strcpy( mMsg, gLabelProperty );
    }
    return S_OK;
  }


  // IPersistPropetyBag. Implement the following functions if you want to save any properties from your overlay.
  // The properties are saved into the Iads Config file in the IadsAVServerPlugins table
	STDMETHOD(GetClassID)(CLSID *pClassID)
	{
  	// IPersist base class method
		*pClassID = this->GetObjectCLSID();
		return S_OK;
	}

	// IPersistPropertyBag
	STDMETHOD(InitNew)()
	{
		return S_OK;  // JFB: Clear dirty flag if needed... Naw, forget it for this simple implementation
	}

  // Here, the AV server is asking you to read all of your known properties from the bag and set your member variables (i.e. it's load/init time)
	STDMETHOD(Load)( IPropertyBag* pPropBag, IErrorLog* pErrorLog )
	{
    if ( !pPropBag ) return E_POINTER;
    CComVariant v; HRESULT hr;

    hr = pPropBag->Read( L"TestProp1", &v, NULL ); if ( hr != S_OK ) return hr;
    wcstombs( mMsg, v.bstrVal, 499 );
		
    return S_OK;
	}

  // Here, the AV server is asking you to save all of your known properties to the bag (i.e. it's save/exit time)
	STDMETHOD(Save)( IPropertyBag* pPropBag, BOOL fClearDirty, BOOL fSaveAllProperties )
	{
    if ( !pPropBag ) return E_POINTER;
    CComVariant v; HRESULT hr;
    
    v = mMsg; hr = pPropBag->Write( L"TestProp1", &v ); if ( hr != S_OK ) return hr;
		
    return S_OK;
	}

};

#endif //__HUDOVERLAY_H_
